@@ -43,7 +43,7 @@ from .._ndvar import neighbor_correlation
 from .._utils.parse import FLOAT_PATTERN, POS_FLOAT_PATTERN, INT_PATTERN
 from .._utils.numpy_utils import FULL_SLICE, INT_TYPES
 from ..mne_fixes import MNE_EPOCHS
-from ..plot._base import AxisData, LayerData, PlotType, AxisScale, find_fig_vlims, find_fig_cmaps
+from ..plot._base import AxisData, DataLayer, PlotType, AxisScale, find_fig_vlims, find_fig_cmaps
 from ..plot._nuts import _plt_bin_nuts
 from ..plot._topo import _ax_topomap
 from ..plot._utsnd import _ax_bfly_epoch
@@ -214,19 +214,14 @@ class Document(FileDocument):
         if trigger in ds:
             trigger = ds[trigger]
         else:
-            err = ("ds does not contain a variable named %r. The trigger "
-                   "parameters needs to point to a variable in ds containing "
-                   "trigger values." % trigger)
-            raise KeyError(err)
+            raise KeyError(f"ds does not contain a variable named {trigger!r}. The trigger parameters needs to point to a variable in ds containing trigger values.")
 
         if INTERPOLATE_CHANNELS in ds:
             interpolate = ds[INTERPOLATE_CHANNELS]
             if not allow_interpolation and any(interpolate):
-                raise ValueError("Dataset contains channel interpolation "
-                                 "information but interpolation is turned off")
+                raise ValueError("Dataset contains channel interpolation information but interpolation is turned off")
         else:
-            interpolate = Datalist([[]] * ds.n_cases, INTERPOLATE_CHANNELS,
-                                   'strlist')
+            interpolate = Datalist([[]] * ds.n_cases, INTERPOLATE_CHANNELS, 'strlist')
 
         if isinstance(blink, str):
             if ds is not None:
@@ -238,17 +233,13 @@ class Document(FileDocument):
                 _, blink = load.eyelink.artifact_epochs(ds, tmin, tmax,
                                                         esacc=False)
             else:
-                msg = ("No eye tracker data was found in ds.info['edf']. Use "
-                       "load.eyelink.add_edf(ds) to add an eye tracker file "
-                       "to a Dataset ds.")
-                wx.MessageBox(msg, "Eye Tracker Data Not Found")
+                wx.MessageBox("No eye tracker data was found in ds.info['edf']. Use load.eyelink.add_edf(ds) to add an eye tracker file to a Dataset ds.", "Eye Tracker Data Not Found")
                 blink = None
         elif blink is not None:
             raise TypeError("blink needs to be a string or None")
 
         if blink is not None:
-            raise NotImplementedError("Frame.SetPage() needs to be updated to "
-                                      "use blink information")
+            raise NotImplementedError("Frame.SetPage() needs to be updated to use blink information")
 
         # options
         self.allow_interpolation = allow_interpolation
@@ -303,8 +294,7 @@ class Document(FileDocument):
                 else:
                     yield epoch
         else:
-            for epoch in epochs:
-                yield epoch
+            yield from epochs
 
     def good_sensor_index(self, case):
         "Index of non-interpolated sensor relative to good sensors"
@@ -385,12 +375,12 @@ class Document(FileDocument):
     ):
         "Read a file making sure it is compatible"
         _, ext = os.path.splitext(path)
-        if ext == '.pickled':
+        if ext.startswith('.pickle'):
             ds = load.unpickle(path)
         elif ext == '.txt':
             ds = load.tsv(path, delimiter='\t')
         else:
-            raise ValueError("Unknown file extension for rejections: %r" % ext)
+            raise ValueError(f"Unknown file extension for rejections: {path}")
 
         # check file contents
         needed = ['trigger', 'accept', 'rej_tag']
@@ -442,8 +432,7 @@ class Document(FileDocument):
                     for l in interpolate:
                         del l[:]
                 else:
-                    raise RuntimeError("File with interpolation when "
-                                       "Interpolation is disabled")
+                    raise RuntimeError("File with interpolation when Interpolation is disabled")
         else:
             interpolate = Datalist([[]] * self.n_epochs, INTERPOLATE_CHANNELS,
                                    'strlist')
@@ -461,8 +450,7 @@ class Document(FileDocument):
 
         # create Dataset to save
         info = {BAD_CHANNELS: self.bad_channel_names}
-        ds = Dataset((self.trigger, self.accept, self.tag, self.interpolate),
-                     info=info)
+        ds = Dataset((self.trigger, self.accept, self.tag, self.interpolate), info=info)
 
         if ext == '.pickled':
             save.pickle(ds, self.path)
@@ -1483,7 +1471,7 @@ class Frame(FileFrame):
                 mark = [ch for ch in self._mark if ch not in self.doc.bad_channel_names]
             else:
                 mark = None
-            layers = AxisData([LayerData(tseg, PlotType.IMAGE)])
+            layers = AxisData([DataLayer(tseg, PlotType.IMAGE)])
             self._topo_plot = _ax_topomap(self._topo_ax, layers, mark=mark, **self._topo_kwargs)
             self._topo_plot_info_str = ""
 
