@@ -1,11 +1,11 @@
 # Author: Christian Brodbeck <christianbrodbeck@nyu.edu>
 from eelbrain import datasets, plot, NDVar
-from eelbrain._wxgui.testing import hide_plots
+from eelbrain.testing import hide_plots
 from eelbrain.plot._glassbrain import _fast_abs_percentile
 
 @hide_plots
 def test_glassbrain():
-    ndvar = datasets.get_mne_stc(True, True)
+    ndvar = datasets.get_mne_stc(True, 'vol-7')
 
     # source space only
     p = plot.GlassBrain(ndvar.source)
